@@ -10,7 +10,9 @@ _ = gpodder.gettext
 
 __title__ = _('Gtk Status Icon')
 __description__ = _('Show a status icon for Gtk-based Desktops.')
+__category__ = 'desktop-integration'
 __only_for__ = 'gtk'
+__disable_in__ = 'unity'
 
 import gtk
 
