@@ -29,15 +29,16 @@ import cgi
 import urllib
 
 class OpmlListModel(gtk.ListStore):
-    C_SELECTED, C_DESCRIPTION_MARKUP, C_URL = range(3)
+    C_SELECTED, C_TITLE, C_DESCRIPTION_MARKUP, C_URL = range(4)
 
     def __init__(self, importer):
-        gtk.ListStore.__init__(self, bool, str, str)
+        gtk.ListStore.__init__(self, bool, str, str, str)
         for channel in importer.items:
-            self.append([False, self._format_channel(channel), channel['url']])
+            self.append([False, channel['title'],
+                self._format_channel(channel), channel['url']])
 
     def _format_channel(self, channel):
         title = cgi.escape(urllib.unquote_plus(channel['title']))
         description = cgi.escape(channel['description'])
-        return '<b>%s</b>\n<span size="small">%s</span>' % (title, description)
+        return '<b>%s</b>\n%s' % (title, description)
 
