@@ -66,6 +66,7 @@ class Store(object):
 
     def close(self):
         with self.lock:
+            self.db.execute('VACUUM')
             self.db.close()
 
     def _register(self, class_):
@@ -88,6 +89,7 @@ class Store(object):
         if isinstance(v, unicode):
             return v
         elif isinstance(v, str):
+            # XXX: Rewrite ^^^ as "isinstance(v, bytes)" in Python 3
             return v.decode('utf-8')
         else:
             return str(v)
