@@ -170,15 +170,6 @@ class PodcastEpisode(PodcastModelObject):
     is_played = property(fget=_deprecated, fset=_deprecated)
     is_locked = property(fget=_deprecated, fset=_deprecated)
 
-    def _get_podcast_id(self):
-        return self.channel.id
-
-    def _set_podcast_id(self, podcast_id):
-        assert self.channel.id == podcast_id
-
-    # Accessor for the "podcast_id" DB column
-    podcast_id = property(fget=_get_podcast_id, fset=_set_podcast_id)
-
     def has_website_link(self):
         return bool(self.link) and (self.link != self.url or \
                 youtube.is_video_link(self.link))
@@ -336,6 +327,7 @@ class PodcastEpisode(PodcastModelObject):
 
     def __init__(self, channel):
         self.parent = channel
+        self.podcast_id = self.parent.id
         self.children = (None, None)
 
         self.id = None
@@ -510,7 +502,7 @@ class PodcastEpisode(PodcastModelObject):
 
         self.set_state(gpodder.STATE_DELETED)
 
-    def get_playback_url(self, fmt_id=None, allow_partial=False):
+    def get_playback_url(self, fmt_ids=None, allow_partial=False):
         """Local (or remote) playback/streaming filename/URL
 
         Returns either the local filename or a streaming URL that
@@ -527,7 +519,7 @@ class PodcastEpisode(PodcastModelObject):
 
         if url is None or not os.path.exists(url):
             url = self.url
-            url = youtube.get_real_download_url(url, fmt_id)
+            url = youtube.get_real_download_url(url, fmt_ids)
             url = vimeo.get_real_download_url(url)
 
         return url
@@ -791,6 +783,15 @@ class PodcastChannel(PodcastModelObject):
 
     UNICODE_TRANSLATE = {ord(u'ö'): u'o', ord(u'ä'): u'a', ord(u'ü'): u'u'}
 
+    # Enumerations for download strategy
+    STRATEGY_DEFAULT, STRATEGY_LATEST = range(2)
+
+    # Description and ordering of strategies
+    STRATEGIES = [
+        (STRATEGY_DEFAULT, _('Default')),
+        (STRATEGY_LATEST, _('Only keep latest')),
+    ]
+
     MAX_FOLDERNAME_LENGTH = 60
     SECONDS_PER_WEEK = 7*24*60*60
     EpisodeClass = PodcastEpisode
@@ -818,9 +819,11 @@ class PodcastChannel(PodcastModelObject):
         self.auto_archive_episodes = False
         self.download_folder = None
         self.pause_subscription = False
+        self.sync_to_mp3_player = True
 
         self.section = _('Other')
         self._common_prefix = None
+        self.download_strategy = PodcastChannel.STRATEGY_DEFAULT
 
     @property
     def model(self):
@@ -830,6 +833,21 @@ class PodcastChannel(PodcastModelObject):
     def db(self):
         return self.parent.db
 
+    def get_download_strategies(self):
+        for value, caption in PodcastChannel.STRATEGIES:
+            yield self.download_strategy == value, value, caption
+
+    def set_download_strategy(self, download_strategy):
+        if download_strategy == self.download_strategy:
+            return
+
+        caption = dict(self.STRATEGIES).get(download_strategy)
+        if caption is not None:
+            logger.debug('Strategy for %s changed to %s', self.title, caption)
+            self.download_strategy = download_strategy
+        else:
+            logger.warn('Cannot set strategy to %d', download_strategy)
+
     def check_download_folder(self):
         """Check the download folder for externally-downloaded files
 
@@ -906,7 +924,7 @@ class PodcastChannel(PodcastModelObject):
                         found = True
                         break
 
-            if not found:
+            if not found and not util.is_system_file(filename):
                 logger.warn('Unknown external file: %s', filename)
 
     @classmethod
@@ -1037,21 +1055,21 @@ class PodcastChannel(PodcastModelObject):
         # Load all episodes to update them properly.
         existing = self.get_all_episodes()
 
-        # We can limit the maximum number of entries that gPodder will parse
-        if max_episodes > 0 and len(feed.entries) > max_episodes:
+        try:
             # We have to sort the entries in descending chronological order,
             # because if the feed lists items in ascending order and has >
             # max_episodes old episodes, new episodes will not be shown.
             # See also: gPodder Bug 1186
-            try:
-                entries = sorted(feed.entries, key=feedcore.get_pubdate,
-                        reverse=True)[:max_episodes]
-            except Exception, e:
-                logger.warn('Could not sort episodes: %s', e, exc_info=True)
-                entries = feed.entries[:max_episodes]
-        else:
+            entries = sorted(feed.entries, key=feedcore.get_pubdate,
+                    reverse=True)
+        except Exception, e:
+            logger.warn('Could not sort episodes: %s', e, exc_info=True)
             entries = feed.entries
 
+        # We can limit the maximum number of entries that gPodder will parse
+        if max_episodes > 0 and len(entries) > max_episodes:
+            entries = entries[:max_episodes]
+
         # Title + PubDate hashes for existing episodes
         existing_dupes = dict((e.duplicate_id(), e) for e in existing)
 
@@ -1064,6 +1082,9 @@ class PodcastChannel(PodcastModelObject):
         # Keep track of episode GUIDs currently seen in the feed
         seen_guids = set()
 
+        # Number of new episodes found
+        new_episodes = 0
+
         # Search all entries for new episodes
         for entry in entries:
             try:
@@ -1104,6 +1125,12 @@ class PodcastChannel(PodcastModelObject):
                     existing_episode.save()
                     continue
 
+            new_episodes += 1
+            # Only allow a certain number of new episodes per update
+            if (self.download_strategy == PodcastChannel.STRATEGY_LATEST and
+                    new_episodes > 1):
+                episode.is_new = False
+
             # Workaround for bug 340: If the episode has been
             # published earlier than one week before the most
             # recent existing episode, do not mark it as new.
