@@ -177,21 +177,18 @@ class Exporter(object):
             # try to save the new file, but keep the old one so we
             # don't end up with a clobbed, empty opml file.
             FREE_DISK_SPACE_AFTER = 1024*512
-            available = util.get_free_disk_space(os.path.dirname(self.filename))
+            path = os.path.dirname(self.filename) or os.path.curdir
+            available = util.get_free_disk_space(path)
             if available < 2*len(data)+FREE_DISK_SPACE_AFTER:
                 # On Windows, if we have zero bytes available, assume that we have
                 # not had the win32file module available + assume enough free space
-                if not gpodder.win32 or available > 0:
+                if not gpodder.ui.win32 or available > 0:
                     logger.error('Not enough free disk space to save channel list to %s', self.filename)
                     return False
             fp = open(self.filename+'.tmp', 'w')
             fp.write(data)
             fp.close()
-            if gpodder.win32:
-                # Win32 does not support atomic rename with os.rename
-                shutil.move(self.filename+'.tmp', self.filename)
-            else:
-                os.rename(self.filename+'.tmp', self.filename)
+            util.atomic_rename(self.filename+'.tmp', self.filename)
         except:
             logger.error('Could not open file for writing: %s', self.filename,
                     exc_info=True)
