@@ -61,9 +61,11 @@ PodcastColumns = (
     'pause_subscription',
     'section',
     'payment_url',
+    'download_strategy',
+    'sync_to_mp3_player',
 )
 
-CURRENT_VERSION = 3
+CURRENT_VERSION = 5
 
 
 # SQL commands to upgrade old database versions to new ones
@@ -82,6 +84,16 @@ UPGRADE_SQL = [
         ALTER TABLE episode ADD COLUMN payment_url TEXT NULL DEFAULT NULL
         UPDATE podcast SET http_last_modified=NULL, http_etag=NULL
         """),
+
+        # Version 4: Per-podcast download strategy management
+        (3, 4, """
+        ALTER TABLE podcast ADD COLUMN download_strategy INTEGER NOT NULL DEFAULT 0
+        """),
+
+        # Version 5: Per-podcast MP3 player device synchronization option
+        (4, 5, """
+        ALTER TABLE podcast ADD COLUMN sync_to_mp3_player INTEGER NOT NULL DEFAULT 1
+        """)
 ]
 
 def initialize_database(db):
@@ -102,7 +114,9 @@ def initialize_database(db):
         download_folder TEXT NOT NULL DEFAULT '',
         pause_subscription INTEGER NOT NULL DEFAULT 0,
         section TEXT NOT NULL DEFAULT '',
-        payment_url TEXT NULL DEFAULT NULL
+        payment_url TEXT NULL DEFAULT NULL,
+        download_strategy INTEGER NOT NULL DEFAULT 0,
+        sync_to_mp3_player INTEGER NOT NULL DEFAULT 1
     )
     """)
 
@@ -224,6 +238,8 @@ def convert_gpodder2_db(old_db, new_db):
                 not row['feed_update_enabled'],
                 '',
                 None,
+                0,
+                row['sync_to_devices'],
         )
         new_db.execute("""
         INSERT INTO podcast VALUES (%s)
